@@ -24,6 +24,7 @@
 #include <VBox/HostServices/VBoxClipboardSvc.h>
 #include "helpers.h"
 #include <iprt/asm.h>
+#include <strsafe.h>
 
 typedef struct _VBOXCLIPBOARDCONTEXT
 {
@@ -81,12 +82,12 @@ static int vboxClipboardConnect (VBOXCLIPBOARDCONTEXT *pCtx)
 
     info.Loc.type = VMMDevHGCMLoc_LocalHost_Existing;
 
-    strcpy (info.Loc.u.host.achName, "VBoxSharedClipboard");
+    memcpy (info.Loc.u.host.achName, "VBoxSharedClipboard", sizeof ("VBoxSharedClipboard"));
 
     DWORD cbReturned;
 
     if (DeviceIoControl (pCtx->pEnv->hDriver,
-                         IOCTL_VBOXGUEST_HGCM_CONNECT,
+                         VBOXGUEST_IOCTL_HGCM_CONNECT,
                          &info, sizeof (info),
                          &info, sizeof (info),
                          &cbReturned,
@@ -121,7 +122,7 @@ static void vboxClipboardDisconnect (VBOXCLIPBOARDCONTEXT *pCtx)
     DWORD cbReturned;
 
     DeviceIoControl (pCtx->pEnv->hDriver,
-                     IOCTL_VBOXGUEST_HGCM_DISCONNECT,
+                     VBOXGUEST_IOCTL_HGCM_DISCONNECT,
                      &info, sizeof (info),
                      &info, sizeof (info),
                      &cbReturned,
@@ -152,7 +153,7 @@ static void VBoxHGCMParmPtrSet (HGCMFunctionParameter *pParm, void *pv, uint32_t
 {
     pParm->type                    = VMMDevHGCMParmType_LinAddr;
     pParm->u.Pointer.size          = cb;
-    pParm->u.Pointer.u.linearAddr  = (RTGCPTR)pv;
+    pParm->u.Pointer.u.linearAddr  = (uintptr_t)pv;
 }
 
 
@@ -161,7 +162,7 @@ static int vboxCall (HANDLE hDriver, void *pvData, unsigned cbData)
     DWORD cbReturned;
 
     if (DeviceIoControl (hDriver,
-                         IOCTL_VBOXGUEST_HGCM_CALL,
+                         VBOXGUEST_IOCTL_HGCM_CALL(cbData),
                          pvData, cbData,
                          pvData, cbData,
                          &cbReturned,
@@ -213,7 +214,7 @@ static int vboxClipboardReadData (VBOXCLIPBOARDCONTEXT *pCtx, uint32_t u32Format
 
             rc = VBoxHGCMParmUInt32Get (&parms.size, &u32Size);
 
-            dprintf (("vboxClipboardReadData: actual size = %d, rc = %d\n", u32Size, rc));
+            Log (("vboxClipboardReadData: actual size = %d, rc = %d\n", u32Size, rc));
 
             if (VBOX_SUCCESS (rc))
             {
@@ -233,7 +234,7 @@ static int vboxClipboardWriteData (VBOXCLIPBOARDCONTEXT *pCtx, uint32_t u32Forma
 {
 //    if (vboxClipboardIsSameAsLastSent (pCtx, u32Format, pv, cb))
 //    {
-//        dprintf (("vboxClipboardWriteData: The data to be sent are the same as the last sent.\n"));
+//        Log (("vboxClipboardWriteData: The data to be sent are the same as the last sent.\n"));
 //        return VINF_SUCCESS;
 //    }
 
@@ -265,7 +266,7 @@ static void vboxClipboardChanged (VBOXCLIPBOARDCONTEXT *pCtx)
 
         while ((format = EnumClipboardFormats (format)) != 0)
         {
-            dprintf (("vboxClipboardChanged: format 0x%08X\n", format));
+            Log (("vboxClipboardChanged: format 0x%08X\n", format));
             switch (format)
             {
                 case CF_UNICODETEXT:
@@ -314,7 +315,7 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
             HWND hwndRemoved = (HWND)wParam;
             HWND hwndNext    = (HWND)lParam;
 
-            dprintf (("vboxClipboardProcessMsg: WM_CHANGECBCHAIN: hwndRemoved %p, hwndNext %p, hwnd %p\n", hwndRemoved, hwndNext, pCtx->hwnd));
+            Log (("vboxClipboardProcessMsg: WM_CHANGECBCHAIN: hwndRemoved %p, hwndNext %p, hwnd %p\n", hwndRemoved, hwndNext, pCtx->hwnd));
 
             if (hwndRemoved == pCtx->hwndNextInChain)
             {
@@ -335,7 +336,7 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
 
         case WM_DRAWCLIPBOARD:
         {
-            dprintf (("vboxClipboardProcessMsg: WM_DRAWCLIPBOARD, hwnd %p\n", pCtx->hwnd));
+            Log (("vboxClipboardProcessMsg: WM_DRAWCLIPBOARD, hwnd %p\n", pCtx->hwnd));
 
             if (GetClipboardOwner () != hwnd)
             {
@@ -359,7 +360,7 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
 
             UINT format = (UINT)wParam;
 
-            dprintf (("vboxClipboardProcessMsg: WM_RENDERFORMAT, format %x\n", format));
+            Log (("vboxClipboardProcessMsg: WM_RENDERFORMAT, format %x\n", format));
             
             switch (format)
             {
@@ -401,40 +402,50 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
                 
                 /* Preallocate a buffer, most of small text transfers will fit into it. */
                 HANDLE hMem = GlobalAlloc (GMEM_DDESHARE | GMEM_MOVEABLE, cbPrealloc);
-                dprintf(("hMem %p\n", hMem));
+                Log(("hMem %p\n", hMem));
 
                 if (hMem)
                 {
                     void *pMem = GlobalLock (hMem);
-                    dprintf(("pMem %p, GlobalSize %d\n", pMem, GlobalSize (hMem)));
+                    Log(("pMem %p, GlobalSize %d\n", pMem, GlobalSize (hMem)));
 
                     if (pMem)
                     {
                         /* Read the host data to the preallocated buffer. */
                         int vboxrc = vboxClipboardReadData (pCtx, u32Format, pMem, cbPrealloc, &cb);
-                        dprintf(("vboxClipboardReadData vboxrc %d\n",  vboxrc));
+                        Log(("vboxClipboardReadData vboxrc %d\n",  vboxrc));
 
-                        if (VBOX_SUCCESS (rc))
+                        if (VBOX_SUCCESS (vboxrc))
                         {
-                            if (cb > cbPrealloc)
+                            if (cb == 0)
+                            {
+                                /* 0 bytes returned means the clipboard is empty.
+                                 * Deallocate the memory and set hMem to NULL to get to
+                                 * the clipboard empty code path.
+                                 */
+                                GlobalUnlock (hMem);
+                                GlobalFree (hMem);
+                                hMem = NULL;
+                            }
+                            else if (cb > cbPrealloc)
                             {
                                 GlobalUnlock (hMem);
                                 
                                 /* The preallocated buffer is too small, adjust the size. */
                                 hMem = GlobalReAlloc (hMem, cb, 0);
-                                dprintf(("hMem %p\n", hMem));
+                                Log(("hMem %p\n", hMem));
                                 
                                 if (hMem)
                                 {
                                     pMem = GlobalLock (hMem);
-                                    dprintf(("pMem %p, GlobalSize %d\n", pMem, GlobalSize (hMem)));
+                                    Log(("pMem %p, GlobalSize %d\n", pMem, GlobalSize (hMem)));
 
                                     if (pMem)
                                     {
                                         /* Read the host data to the preallocated buffer. */
                                         uint32_t cbNew = 0;
                                         vboxrc = vboxClipboardReadData (pCtx, u32Format, pMem, cb, &cbNew);
-                                        dprintf(("vboxClipboardReadData vboxrc %d, cb = %d, cbNew = %d\n", vboxrc, cb, cbNew));
+                                        Log(("vboxClipboardReadData vboxrc %d, cb = %d, cbNew = %d\n", vboxrc, cb, cbNew));
 
                                         if (VBOX_SUCCESS (vboxrc) && cbNew <= cb)
                                         {
@@ -442,7 +453,7 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
                                         }
                                         else
                                         {
-                                            GlobalUnlock (pMem);
+                                            GlobalUnlock (hMem);
                                             GlobalFree (hMem);
                                             hMem = NULL;
                                         }
@@ -458,16 +469,36 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
                             if (hMem)
                             {
                                 /* pMem is the address of the data. cb is the size of returned data. */
-                                /* Verify the size of returned text. */
+                                /* Verify the size of returned text, the memory block for clipboard
+                                 * must have the exact string size.
+                                 */
                                 if (u32Format == VBOX_SHARED_CLIPBOARD_FMT_UNICODETEXT)
                                 {
-                                    cb = (lstrlenW((LPWSTR)pMem) + 1) * 2;
+                                    size_t cbActual = 0;
+                                    HRESULT hrc = StringCbLengthW ((LPWSTR)pMem, cb, &cbActual);
+                                    if (FAILED (hrc))
+                                    {
+                                        /* Discard invalid data. */
+                                        GlobalUnlock (hMem);
+                                        GlobalFree (hMem);
+                                        hMem = NULL;
+                                    }
+                                    else
+                                    {
+                                        /* cbActual is the number of bytes, excluding those used
+                                         * for the terminating null character.
+                                         */
+                                        cb = (uint32_t)(cbActual + 2);
+                                    }
                                 }
+                            }
                                 
+                            if (hMem)
+                            {
                                 GlobalUnlock (hMem);
                                 
                                 hMem = GlobalReAlloc (hMem, cb, 0);
-                                dprintf(("hMem %p\n", hMem));
+                                Log(("hMem %p\n", hMem));
                                 
                                 if (hMem)
                                 {
@@ -475,7 +506,7 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
                                      * size is 'cb' and format is 'format'.
                                      */
                                     HANDLE hClip = SetClipboardData (format, hMem);
-                                    dprintf(("WM_RENDERFORMAT hClip %p\n", hClip));
+                                    Log(("WM_RENDERFORMAT hClip %p\n", hClip));
 
                                     if (hClip)
                                     {
@@ -531,14 +562,14 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
 
                 if (u32Formats & VBOX_SHARED_CLIPBOARD_FMT_UNICODETEXT)
                 {
-                    dprintf(("window proc WM_USER: VBOX_SHARED_CLIPBOARD_FMT_UNICODETEXT\n"));
+                    Log(("window proc WM_USER: VBOX_SHARED_CLIPBOARD_FMT_UNICODETEXT\n"));
                     
                     hClip = SetClipboardData (CF_UNICODETEXT, NULL);
                 }
 
                 if (u32Formats & VBOX_SHARED_CLIPBOARD_FMT_BITMAP)
                 {
-                    dprintf(("window proc WM_USER: VBOX_SHARED_CLIPBOARD_FMT_BITMAP\n"));
+                    Log(("window proc WM_USER: VBOX_SHARED_CLIPBOARD_FMT_BITMAP\n"));
                     
                     hClip = SetClipboardData (CF_DIB, NULL);
                 }
@@ -546,7 +577,7 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
                 if (u32Formats & VBOX_SHARED_CLIPBOARD_FMT_HTML)
                 {
                     UINT format = RegisterClipboardFormat ("HTML Format");
-                    dprintf(("window proc WM_USER: VBOX_SHARED_CLIPBOARD_FMT_HTML 0x%04X\n", format));
+                    Log(("window proc WM_USER: VBOX_SHARED_CLIPBOARD_FMT_HTML 0x%04X\n", format));
                     if (format != 0)
                     {
                         hClip = SetClipboardData (format, NULL);
@@ -555,11 +586,11 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
 
                 CloseClipboard();
 
-                dprintf(("window proc WM_USER: hClip %p, err %d\n", hClip, GetLastError ()));
+                Log(("window proc WM_USER: hClip %p, err %d\n", hClip, GetLastError ()));
             }
             else
             {
-                dprintf(("window proc WM_USER: failed to open clipboard\n"));
+                Log(("window proc WM_USER: failed to open clipboard\n"));
             }
         } break;
 
@@ -582,7 +613,7 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
 
                         if (lp != NULL)
                         {
-                            dprintf(("CF_DIB\n"));
+                            Log(("CF_DIB\n"));
 
                             vboxClipboardWriteData (pCtx, VBOX_SHARED_CLIPBOARD_FMT_BITMAP,
                                                     lp, GlobalSize (hClip));
@@ -605,7 +636,7 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
 
                         if (uniString != NULL)
                         {
-                            dprintf(("CF_UNICODETEXT\n"));
+                            Log(("CF_UNICODETEXT\n"));
 
                             vboxClipboardWriteData (pCtx, VBOX_SHARED_CLIPBOARD_FMT_UNICODETEXT,
                                                     uniString, (lstrlenW (uniString) + 1) * 2);
@@ -632,7 +663,7 @@ static LRESULT vboxClipboardProcessMsg(VBOXCLIPBOARDCONTEXT *pCtx, HWND hwnd, UI
 
                             if (lp != NULL)
                             {
-                                dprintf(("CF_HTML\n"));
+                                Log(("CF_HTML\n"));
 
                                 vboxClipboardWriteData (pCtx, VBOX_SHARED_CLIPBOARD_FMT_HTML,
                                                         lp, GlobalSize (hClip));
@@ -747,7 +778,7 @@ int VBoxClipboardInit (const VBOXSERVICEENV *pEnv, void **ppInstance, bool *pfSt
 {
     int rc = VINF_SUCCESS;
 
-    dprintf (("VboxClipboardInit\n"));
+    Log (("VboxClipboardInit\n"));
 
     if (gCtx.pEnv)
     {
@@ -765,7 +796,7 @@ int VBoxClipboardInit (const VBOXSERVICEENV *pEnv, void **ppInstance, bool *pfSt
     {
         rc = vboxClipboardInit (&gCtx);
 
-        dprintf (("vboxClipboardInit: rc = %d\n", rc));
+        Log (("vboxClipboardInit: rc = %d\n", rc));
 
         if (VBOX_SUCCESS (rc))
         {
@@ -790,7 +821,7 @@ unsigned __stdcall VBoxClipboardThread (void *pInstance)
 {
     VBOXCLIPBOARDCONTEXT *pCtx = (VBOXCLIPBOARDCONTEXT *)pInstance;
 
-    dprintf(("VBoxClipboardThread\n"));
+    Log(("VBoxClipboardThread\n"));
     
     /* Open the new driver instance to not interfere with other callers. */
     HANDLE hDriver = CreateFile(VBOXGUEST_DEVICE_NAME,
@@ -814,13 +845,13 @@ unsigned __stdcall VBoxClipboardThread (void *pInstance)
         DWORD cbReturned;
 
         if (!DeviceIoControl (hDriver,
-                              IOCTL_VBOXGUEST_HGCM_CALL,
+                              VBOXGUEST_IOCTL_HGCM_CALL(sizeof (parms)),
                               &parms, sizeof (parms),
                               &parms, sizeof (parms),
                               &cbReturned,
                               NULL))
         {
-            dprintf(("Failed to call the driver for host message.\n"));
+            Log(("Failed to call the driver for host message.\n"));
 
             /* Wait a bit before retrying. */
             if (WaitForSingleObject(pCtx->pEnv->hStopEvent, 1000) == WAIT_OBJECT_0)
@@ -846,7 +877,7 @@ unsigned __stdcall VBoxClipboardThread (void *pInstance)
 
                 if (VBOX_SUCCESS (rc))
                 {
-                    dprintf(("vboxClipboardHostEvent u32Msg %d, u32Formats %d\n", u32Msg, u32Formats));
+                    Log(("vboxClipboardHostEvent u32Msg %d, u32Formats %d\n", u32Msg, u32Formats));
     
                     switch (u32Msg)
                     {
@@ -872,7 +903,7 @@ unsigned __stdcall VBoxClipboardThread (void *pInstance)
                         } break;
                         default:
                         {
-                            dprintf(("Unsupported message from host!!!"));
+                            Log(("Unsupported message from host!!!\n"));
                         }
                     }
                 }
@@ -897,7 +928,7 @@ unsigned __stdcall VBoxClipboardThread (void *pInstance)
             continue;
         }
 
-        dprintf(("processed host event rc = %d\n", rc));
+        Log(("processed host event rc = %d\n", rc));
     }
     
     CloseHandle (hDriver);
@@ -911,7 +942,7 @@ void VBoxClipboardDestroy (const VBOXSERVICEENV *pEnv, void *pInstance)
 
     if (pCtx != &gCtx)
     {
-        dprintf(("VBoxClipboardDestroy: invalid instance %p (our %p)!!!\n", pCtx, &gCtx));
+        Log(("VBoxClipboardDestroy: invalid instance %p (our %p)!!!\n", pCtx, &gCtx));
 
         pCtx = &gCtx;
     }
